import csv
import datetime
from django.http import HttpResponse, HttpResponseForbidden
from django.shortcuts import get_object_or_404, render, redirect
from django.contrib import messages
from django.core.paginator import Paginator
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ObjectDoesNotExist

from .models import Driver, Vehicle

@login_required
def create_driver(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    if request.method == 'POST':
        name = request.POST.get('name')
        telephone = request.POST.get('telephone')
        driver_id = request.POST.get('driver_id')
        licensed = request.POST.get('licensed') == 'on'
        vehicle_id = request.POST.get('vehicle_assigned')

        if Driver.objects.filter(driver_id=driver_id).exists():
            messages.error(request, 'A driver with this ID already exists.')
            return redirect('backoffice:create_driver')

        vehicle = None
        if vehicle_id:
            vehicle = get_object_or_404(Vehicle, pk=vehicle_id)

        try:
            Driver.objects.create(
                name=name,
                telephone=telephone,
                driver_id=driver_id,
                licensed=licensed,
                vehicle_assigned=vehicle
            )
            messages.success(request, 'Driver added successfully.')
            return redirect('backoffice:driver_list')
        except Exception as e:
            messages.error(request, f'An error occurred: {str(e)}')

    vehicles = Vehicle.objects.all()
    return render(request, 'backoffice/transport/create_transport.html', {'vehicles': vehicles})

@login_required
def create_vehicle(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    if request.method == 'POST':
        car_number = request.POST.get('car_number')
        transport_mode = request.POST.get('transport_mode')
        capacity = request.POST.get('capacity')

        if Vehicle.objects.filter(car_number=car_number).exists():
            messages.error(request, 'A vehicle with this car number already exists.')
            return redirect('backoffice:driver_list')

        try:
            Vehicle.objects.create(
                car_number=car_number,
                transport_mode=transport_mode,
                capacity=capacity
            )
            messages.success(request, 'Vehicle added successfully.')
            return redirect('backoffice:driver_list')
        except Exception as e:
            messages.error(request, f'An error occurred: {str(e)}')

    return render(request, 'backoffice/transport/create_transport.html')

@login_required
def driver_list(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    search_query = request.GET.get('search_query', '')
    start_date = request.GET.get('start_date', '')
    end_date = request.GET.get('end_date', '')

    drivers_query = Driver.objects.all().order_by('-name')

    if search_query:
        drivers_query = drivers_query.filter(name__icontains=search_query)

    if start_date and end_date:
        start_date_obj = datetime.datetime.strptime(start_date, "%Y-%m-%d").date()
        end_date_obj = datetime.datetime.strptime(end_date, "%Y-%m-%d").date()
        drivers_query = drivers_query.filter(created_at__date__gte=start_date_obj, created_at__date__lte=end_date_obj)

    paginator = Paginator(drivers_query, 10)
    page_number = request.GET.get('page')
    drivers = paginator.get_page(page_number)

    vehicles = Vehicle.objects.all()

    context = {
        'drivers': drivers,
        'vehicles': vehicles,
        'search_query': search_query,
        'start_date': start_date,
        'end_date': end_date
    }
    return render(request, 'backoffice/transport/create_transport.html', context)

@login_required
def export_drivers_excel(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="drivers.csv"'

    writer = csv.writer(response)
    writer.writerow(['Driver Name', 'Telephone', 'Driver ID', 'Licensed', 'Vehicle Assigned'])

    drivers = Driver.objects.all().order_by('-name')
    for driver in drivers:
        writer.writerow([
            driver.name,
            driver.telephone,
            driver.driver_id,
            "Yes" if driver.licensed else "No",
            driver.vehicle_assigned.car_number if driver.vehicle_assigned else "Not Assigned"
        ])

    return response

@login_required
def delete_driver(request, driver_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        driver = Driver.objects.get(id=driver_id)
        driver.delete()
        messages.success(request, 'Driver deleted successfully.')
    except ObjectDoesNotExist:
        messages.error(request, 'Driver not found.')
    except Exception as e:
        messages.error(request, f'An error occurred: {str(e)}')

    return redirect('backoffice:driver_list')

@login_required
def update_driver(request, driver_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    driver = get_object_or_404(Driver, id=driver_id)
    vehicles = Vehicle.objects.all()

    if request.method == 'POST':
        try:
            driver.name = request.POST.get('name')
            driver.telephone = request.POST.get('telephone')
            driver.driver_id = request.POST.get('driver_id')
            driver.licensed = request.POST.get('licensed') == 'on'
            vehicle_id = request.POST.get('vehicle_assigned')

            if vehicle_id:
                vehicle = Vehicle.objects.get(id=vehicle_id)
                driver.vehicle_assigned = vehicle
            else:
                driver.vehicle_assigned = None

            driver.save()
            messages.success(request, 'Driver updated successfully.')
            return redirect('backoffice:driver_list')
        except Exception as e:
            messages.error(request, f'An error occurred: {str(e)}')

    context = {
        'driver': driver,
        'vehicles': vehicles
    }
    return render(request, 'backoffice/transport/create_transport.html', context)

@login_required
def vehicle_list(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    vehicles = Vehicle.objects.all().order_by('-car_number')
    paginator = Paginator(vehicles, 10)
    page_number = request.GET.get('page')
    vehicles = paginator.get_page(page_number)

    return render(request, 'backoffice/transport/create_transport.html', {'vehicles': vehicles})

@login_required
def update_vehicle(request, vehicle_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    vehicle = get_object_or_404(Vehicle, id=vehicle_id)

    if request.method == 'POST':
        try:
            vehicle.car_number = request.POST.get('car_number')
            vehicle.transport_mode = request.POST.get('transport_mode')
            vehicle.capacity = request.POST.get('capacity')
            vehicle.save()
            messages.success(request, 'Vehicle updated successfully.')
            return redirect('backoffice:vehicle_list')
        except Exception as e:
            messages.error(request, f'An error occurred: {str(e)}')

    return render(request, 'backoffice/transport/create_transport.html', {'vehicle': vehicle})

@login_required
def delete_vehicle(request, vehicle_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    try:
        vehicle = Vehicle.objects.get(id=vehicle_id)
        vehicle.delete()
        messages.success(request, 'Vehicle deleted successfully.')
    except ObjectDoesNotExist:
        messages.error(request, 'Vehicle not found.')
    except Exception as e:
        messages.error(request, f'An error occurred: {str(e)}')

    return redirect('backoffice:vehicle_list')